# coding: utf-8

"""
Marketcap Service API

API for retrieving historical marketcap data, available exchanges, and indicators.

The version of the OpenAPI document: 1.0.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

import io
import json
import re
import ssl
from typing import Optional, Union

import aiohttp
import aiohttp_retry

from crypticorn.metrics.client.exceptions import ApiException, ApiValueError

RESTResponseType = aiohttp.ClientResponse

ALLOW_RETRY_METHODS = frozenset({"DELETE", "GET", "HEAD", "OPTIONS", "PUT", "TRACE"})


class RESTResponse(io.IOBase):
    def __init__(self, resp) -> None:
        self.response = resp
        self.status = resp.status
        self.reason = resp.reason
        self.data = None

    async def read(self):
        if self.data is None:
            self.data = await self.response.read()
        return self.data

    def getheaders(self):
        """Returns a CIMultiDictProxy of the response headers."""
        return self.response.headers

    def getheader(self, name, default=None):
        """Returns a given response header."""
        return self.response.headers.get(name, default)


class RESTClientObject:
    def __init__(self, configuration) -> None:
        # maxsize is number of requests to host that are allowed in parallel
        self.maxsize = configuration.connection_pool_maxsize

        self.ssl_context = ssl.create_default_context(
            cafile=configuration.ssl_ca_cert,
            cadata=configuration.ca_cert_data,
        )
        if configuration.cert_file:
            self.ssl_context.load_cert_chain(
                configuration.cert_file, keyfile=configuration.key_file
            )

        if not configuration.verify_ssl:
            self.ssl_context.check_hostname = False
            self.ssl_context.verify_mode = ssl.CERT_NONE

        self.proxy = configuration.proxy
        self.proxy_headers = configuration.proxy_headers

        self.retries = configuration.retries

        self.pool_manager: Optional[aiohttp.ClientSession] = None
        self.retry_client: Optional[aiohttp_retry.RetryClient] = None
        self.is_sync: bool = False  # Track whether this is sync or async mode

    async def close(self) -> None:
        if self.pool_manager:
            await self.pool_manager.close()
        if self.retry_client is not None:
            await self.retry_client.close()

    async def request(
        self,
        method,
        url,
        headers=None,
        body=None,
        post_params=None,
        _request_timeout=None,
    ):
        """Execute request

        :param method: http request method
        :param url: http request url
        :param headers: http request headers
        :param body: request json body, for `application/json`
        :param post_params: request post parameters,
                            `application/x-www-form-urlencoded`
                            and `multipart/form-data`
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        """
        method = method.upper()
        assert method in ["GET", "HEAD", "DELETE", "POST", "PUT", "PATCH", "OPTIONS"]

        if post_params and body:
            raise ApiValueError(
                "body parameter cannot be used with post_params parameter."
            )

        post_params = post_params or {}
        headers = headers or {}
        # url already contains the URL query string
        timeout = _request_timeout or 5 * 60

        if "Content-Type" not in headers:
            headers["Content-Type"] = "application/json"

        args = {"method": method, "url": url, "timeout": timeout, "headers": headers}

        if self.proxy:
            args["proxy"] = self.proxy
        if self.proxy_headers:
            args["proxy_headers"] = self.proxy_headers

        # For `POST`, `PUT`, `PATCH`, `OPTIONS`, `DELETE`
        if method in ["POST", "PUT", "PATCH", "OPTIONS", "DELETE"]:
            if re.search("json", headers["Content-Type"], re.IGNORECASE):
                if body is not None:
                    body = json.dumps(body)
                args["data"] = body
            elif headers["Content-Type"] == "application/x-www-form-urlencoded":
                args["data"] = aiohttp.FormData(post_params)
            elif headers["Content-Type"] == "multipart/form-data":
                # must del headers['Content-Type'], or the correct
                # Content-Type which generated by aiohttp
                del headers["Content-Type"]
                data = aiohttp.FormData()
                for param in post_params:
                    k, v = param
                    if isinstance(v, tuple) and len(v) == 3:
                        data.add_field(k, value=v[1], filename=v[0], content_type=v[2])
                    else:
                        # Ensures that dict objects are serialized
                        if isinstance(v, dict):
                            v = json.dumps(v)
                        elif isinstance(v, int):
                            v = str(v)
                        data.add_field(k, v)
                args["data"] = data

            # Pass a `bytes` or `str` parameter directly in the body to support
            # other content types than Json when `body` argument is provided
            # in serialized form
            elif isinstance(body, str) or isinstance(body, bytes):
                args["data"] = body
            else:
                # Cannot generate the request from given parameters
                msg = """Cannot prepare a request message for provided
                         arguments. Please check that your arguments match
                         declared content type."""
                raise ApiException(status=0, reason=msg)

        pool_manager: Union[aiohttp.ClientSession, aiohttp_retry.RetryClient]

        # For sync operations, always use a fresh session
        should_close_session = False

        if self.pool_manager is None:
            self.pool_manager = aiohttp.ClientSession(
                connector=aiohttp.TCPConnector(
                    limit=self.maxsize, ssl=self.ssl_context
                ),
                trust_env=True,
            )
            # Only close session automatically in sync mode
            should_close_session = self.is_sync

        pool_manager = self.pool_manager

        if self.retries is not None and method in ALLOW_RETRY_METHODS:
            if self.retry_client is None:
                self.retry_client = aiohttp_retry.RetryClient(
                    client_session=self.pool_manager,
                    retry_options=aiohttp_retry.ExponentialRetry(
                        attempts=self.retries,
                        factor=2.0,
                        start_timeout=0.1,
                        max_timeout=120.0,
                    ),
                )
            pool_manager = self.retry_client

        try:
            r = await pool_manager.request(**args)
            # For sessions we're about to close, read the data immediately
            if should_close_session:
                response = RESTResponse(r)
                await response.read()  # Read data before closing session
                return response
            else:
                return RESTResponse(r)
        finally:
            if should_close_session:
                if self.retry_client is not None:
                    await self.retry_client.close()
                    self.retry_client = None
                await self.pool_manager.close()
                self.pool_manager = None



# coding: utf-8

"""
Notifications API

API for notification delivery system

The version of the OpenAPI document: 1.0.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

from __future__ import annotations

import json
import pprint
import re  # noqa: F401
from typing import Any, ClassVar, Dict, List, Optional, Set

from pydantic import BaseModel, ConfigDict, Field, StrictInt, StrictStr
from typing_extensions import Annotated, Self

from crypticorn.notification.client.models.broadcast_recipient import BroadcastRecipient


class Broadcast(BaseModel):
    """
    Broadcast with communication preferences
    """  # noqa: E501

    id: StrictStr = Field(description="Unique identifier for the resource")
    created_at: StrictInt = Field(description="Timestamp of creation")
    updated_at: StrictInt = Field(description="Timestamp of last update")
    template_preferences: List[
        Annotated[List[Any], Field(min_length=2, max_length=2)]
    ] = Field(
        description="List of templates and whether the broadcast sends the notification for"
    )
    recipients: List[BroadcastRecipient] = Field(
        description="List of recipients for the broadcast to be sent to"
    )
    __properties: ClassVar[List[str]] = [
        "id",
        "created_at",
        "updated_at",
        "template_preferences",
        "recipients",
    ]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of Broadcast from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in recipients (list)
        _items = []
        if self.recipients:
            for _item_recipients in self.recipients:
                if _item_recipients:
                    _items.append(_item_recipients.to_dict())
            _dict["recipients"] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of Broadcast from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "id": obj.get("id"),
                "created_at": obj.get("created_at"),
                "updated_at": obj.get("updated_at"),
                "template_preferences": obj.get("template_preferences"),
                "recipients": (
                    [BroadcastRecipient.from_dict(_item) for _item in obj["recipients"]]
                    if obj.get("recipients") is not None
                    else None
                ),
            }
        )
        return _obj



from __future__ import annotations

import warnings
from typing import TYPE_CHECKING, Optional

from typing_extensions import deprecated

from crypticorn._internal.warnings import CrypticornDeprecatedSince31
from crypticorn.metrics import (
    ApiClient,
    Configuration,
    IndicatorsApi,
    LogsApi,
    MarketsApi,
    QuoteCurrenciesApi,
    StatusApi,
)
from crypticorn.metrics.wrapper import ExchangesApiWrapper  # wraps ExchangesApi
from crypticorn.metrics.wrapper import MarketcapApiWrapper  # wraps MarketcapApi
from crypticorn.metrics.wrapper import TokensApiWrapper  # wraps TokensApi

if TYPE_CHECKING:
    from aiohttp import ClientSession


class MetricsClient(
    ExchangesApiWrapper,
    IndicatorsApi,
    LogsApi,
    MarketcapApiWrapper,
    MarketsApi,
    QuoteCurrenciesApi,
    StatusApi,
    TokensApiWrapper,
):
    """
    A client for interacting with the Crypticorn Metrics API.
    """

    config_class = Configuration

    def __init__(
        self,
        config: Configuration,
        http_client: Optional[ClientSession] = None,
        is_sync: bool = False,
    ):
        self.config = config
        self.base_client = ApiClient(configuration=self.config)
        if http_client is not None:
            self.base_client.rest_client.pool_manager = http_client
        # Pass sync context to REST client for proper session management
        self.base_client.rest_client.is_sync = is_sync
        super().__init__(self.base_client, is_sync=is_sync)
        # TODO: remove everything below this line in v4
        self._status = StatusApi(self.base_client, is_sync=is_sync)
        self._indicators = IndicatorsApi(self.base_client, is_sync=is_sync)
        self._logs = LogsApi(self.base_client, is_sync=is_sync)
        self._marketcap = MarketcapApiWrapper(self.base_client, is_sync=is_sync)
        self._markets = MarketsApi(self.base_client, is_sync=is_sync)
        self._tokens = TokensApiWrapper(self.base_client, is_sync=is_sync)
        self._exchanges = ExchangesApiWrapper(self.base_client, is_sync=is_sync)
        self._quote_currencies = QuoteCurrenciesApi(self.base_client, is_sync=is_sync)

    @property
    @deprecated(
        "Accessing metrics.status is deprecated. Use direct method calls instead (e.g., metrics.ping())"
    )
    def status(self):
        warnings.warn(
            "Accessing metrics.status is deprecated. Use direct method calls instead (e.g., metrics.ping())",
            category=CrypticornDeprecatedSince31,
        )
        return self._status

    @property
    @deprecated(
        "Accessing metrics.indicators is deprecated. Use direct method calls instead (e.g., metrics.get_indicators())"
    )
    def indicators(self):
        warnings.warn(
            "Accessing metrics.indicators is deprecated. Use direct method calls instead (e.g., metrics.get_indicators())",
            category=CrypticornDeprecatedSince31,
        )
        return self._indicators

    @property
    @deprecated(
        "Accessing metrics.logs is deprecated. Use direct method calls instead (e.g., metrics.get_logs())"
    )
    def logs(self):
        warnings.warn(
            "Accessing metrics.logs is deprecated. Use direct method calls instead (e.g., metrics.get_logs())",
            category=CrypticornDeprecatedSince31,
        )
        return self._logs

    @property
    @deprecated(
        "Accessing metrics.marketcap is deprecated. Use direct method calls instead (e.g., metrics.get_marketcap_symbols_fmt())"
    )
    def marketcap(self):
        warnings.warn(
            "Accessing metrics.marketcap is deprecated. Use direct method calls instead (e.g., metrics.get_marketcap_symbols_fmt())",
            category=CrypticornDeprecatedSince31,
        )
        return self._marketcap

    @property
    @deprecated(
        "Accessing metrics.markets is deprecated. Use direct method calls instead (e.g., metrics.get_markets())"
    )
    def markets(self):
        warnings.warn(
            "Accessing metrics.markets is deprecated. Use direct method calls instead (e.g., metrics.get_markets())",
            category=CrypticornDeprecatedSince31,
        )
        return self._markets

    @property
    @deprecated(
        "Accessing metrics.tokens is deprecated. Use direct method calls instead (e.g., metrics.get_stable_tokens_fmt())"
    )
    def tokens(self):
        warnings.warn(
            "Accessing metrics.tokens is deprecated. Use direct method calls instead (e.g., metrics.get_stable_tokens_fmt())",
            category=CrypticornDeprecatedSince31,
        )
        return self._tokens

    @property
    @deprecated(
        "Accessing metrics.exchanges is deprecated. Use direct method calls instead (e.g., metrics.get_available_exchanges_fmt())"
    )
    def exchanges(self):
        warnings.warn(
            "Accessing metrics.exchanges is deprecated. Use direct method calls instead (e.g., metrics.get_available_exchanges_fmt())",
            category=CrypticornDeprecatedSince31,
        )
        return self._exchanges

    @property
    @deprecated(
        "Accessing metrics.quote_currencies is deprecated. Use direct method calls instead (e.g., metrics.get_quote_currencies())"
    )
    def quote_currencies(self):
        warnings.warn(
            "Accessing metrics.quote_currencies is deprecated. Use direct method calls instead (e.g., metrics.get_quote_currencies())",
            category=CrypticornDeprecatedSince31,
        )
        return self._quote_currencies



from __future__ import annotations

from typing import TYPE_CHECKING, Optional

from crypticorn.trade.client import (
    ApiClient,
    APIKeysApi,
    BotsApi,
    Configuration,
    StatusApi,
    StrategiesApi,
    TradingActionsApi,
)

if TYPE_CHECKING:
    from aiohttp import ClientSession


class TradeClient(BotsApi, StatusApi, StrategiesApi, TradingActionsApi, APIKeysApi):
    """
    A client for interacting with the Crypticorn Trade API.
    """

    config_class = Configuration

    def __init__(
        self,
        config: Configuration,
        http_client: Optional[ClientSession] = None,
        is_sync: bool = False,
    ):
        self.config = config
        self.base_client = ApiClient(configuration=self.config)
        if http_client is not None:
            self.base_client.rest_client.pool_manager = http_client
        # Pass sync context to REST client for proper session management
        self.base_client.rest_client.is_sync = is_sync
        # Instantiate all the endpoint clients
        self.bots = BotsApi(self.base_client, is_sync=is_sync)
        self.status = StatusApi(self.base_client, is_sync=is_sync)
        self.strategies = StrategiesApi(self.base_client, is_sync=is_sync)
        self.actions = TradingActionsApi(self.base_client, is_sync=is_sync)
        self.keys = APIKeysApi(self.base_client, is_sync=is_sync)



from __future__ import annotations

import warnings
from typing import TYPE_CHECKING, Optional

from typing_extensions import deprecated

from crypticorn._internal.warnings import CrypticornDeprecatedSince31
from crypticorn.pay import (
    AccessApi,
    ApiClient,
    Configuration,
    CouponsApi,
    InvoicesApi,
    NOWPaymentsApi,
    PaymentsApi,
    ProductsApi,
    StatusApi,
    StripeApi,
    TokenApi,
)

if TYPE_CHECKING:
    from aiohttp import ClientSession


class PayClient(
    AccessApi,
    CouponsApi,
    InvoicesApi,
    NOWPaymentsApi,
    PaymentsApi,
    ProductsApi,
    StatusApi,
    StripeApi,
    TokenApi,
):
    """
    A client for interacting with the Crypticorn Pay API.
    """

    config_class = Configuration

    def __init__(
        self,
        config: Configuration,
        http_client: Optional[ClientSession] = None,
        is_sync: bool = False,
    ):
        self.config = config
        self.base_client = ApiClient(configuration=self.config)
        if http_client is not None:
            self.base_client.rest_client.pool_manager = http_client
        # Pass sync context to REST client for proper session management
        self.base_client.rest_client.is_sync = is_sync
        super().__init__(self.base_client, is_sync=is_sync)
        # TODO: remove everything below this line in v4
        self._now = NOWPaymentsApi(self.base_client, is_sync=is_sync)
        self._status = StatusApi(self.base_client, is_sync=is_sync)
        self._payments = PaymentsApi(self.base_client, is_sync=is_sync)
        self._products = ProductsApi(self.base_client, is_sync=is_sync)
        self._coupons = CouponsApi(self.base_client, is_sync=is_sync)
        self._token = TokenApi(self.base_client, is_sync=is_sync)
        self._invoices = InvoicesApi(self.base_client, is_sync=is_sync)
        self._stripe = StripeApi(self.base_client, is_sync=is_sync)
        self._access = AccessApi(self.base_client, is_sync=is_sync)

    @property
    @deprecated(
        "Accessing pay.now is deprecated. Use direct method calls on pay instead (e.g., pay.create_now_payment())"
    )
    def now(self):
        warnings.warn(
            "Accessing pay.now is deprecated. Use direct method calls on pay instead (e.g., pay.create_now_payment())",
            category=CrypticornDeprecatedSince31,
        )
        return self._now

    @property
    @deprecated(
        "Accessing pay.status is deprecated. Use direct method calls on pay instead (e.g., pay.ping())"
    )
    def status(self):
        warnings.warn(
            "Accessing pay.status is deprecated. Use direct method calls on pay instead (e.g., pay.ping())",
            category=CrypticornDeprecatedSince31,
        )
        return self._status

    @property
    @deprecated(
        "Accessing pay.payments is deprecated. Use direct method calls on pay instead (e.g., pay.get_payments())"
    )
    def payments(self):
        warnings.warn(
            "Accessing pay.payments is deprecated. Use direct method calls on pay instead (e.g., pay.get_payments())",
            category=CrypticornDeprecatedSince31,
        )
        return self._payments

    @property
    @deprecated(
        "Accessing pay.products is deprecated. Use direct method calls on pay instead (e.g., pay.get_products())"
    )
    def products(self):
        warnings.warn(
            "Accessing pay.products is deprecated. Use direct method calls on pay instead (e.g., pay.get_products())",
            category=CrypticornDeprecatedSince31,
        )
        return self._products

    @property
    @deprecated(
        "Accessing pay.coupons is deprecated. Use direct method calls on pay instead (e.g., pay.get_coupons())"
    )
    def coupons(self):
        warnings.warn(
            "Accessing pay.coupons is deprecated. Use direct method calls on pay instead (e.g., pay.get_coupons())",
            category=CrypticornDeprecatedSince31,
        )
        return self._coupons

    @property
    @deprecated(
        "Accessing pay.token is deprecated. Use direct method calls on pay instead (e.g., pay.get_token())"
    )
    def token(self):
        warnings.warn(
            "Accessing pay.token is deprecated. Use direct method calls on pay instead (e.g., pay.get_token())",
            category=CrypticornDeprecatedSince31,
        )
        return self._token

    @property
    @deprecated(
        "Accessing pay.invoices is deprecated. Use direct method calls on pay instead (e.g., pay.get_invoices())"
    )
    def invoices(self):
        warnings.warn(
            "Accessing pay.invoices is deprecated. Use direct method calls on pay instead (e.g., pay.get_invoices())",
            category=CrypticornDeprecatedSince31,
        )
        return self._invoices

    @property
    @deprecated(
        "Accessing pay.stripe is deprecated. Use direct method calls on pay instead (e.g., pay.create_stripe_session())"
    )
    def stripe(self):
        warnings.warn(
            "Accessing pay.stripe is deprecated. Use direct method calls on pay instead (e.g., pay.create_stripe_session())",
            category=CrypticornDeprecatedSince31,
        )
        return self._stripe

    @property
    @deprecated(
        "Accessing pay.access is deprecated. Use direct method calls on pay instead (e.g., pay.get_access_scopes())"
    )
    def access(self):
        warnings.warn(
            "Accessing pay.access is deprecated. Use direct method calls on pay instead (e.g., pay.get_access_scopes())",
            category=CrypticornDeprecatedSince31,
        )
        return self._access



# coding: utf-8

"""
Payment API

API for accepting payments and storing subscriptions

The version of the OpenAPI document: 1.0.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

from __future__ import annotations

import json
import pprint
import re  # noqa: F401
from typing import Any, ClassVar, Dict, List, Optional, Set

from pydantic import BaseModel, ConfigDict, StrictInt, StrictStr
from typing_extensions import Self


class CouponUpdate(BaseModel):
    """
    Model for updating a coupon
    """  # noqa: E501

    valid_until: Optional[StrictInt] = None
    usage_limit: Optional[StrictInt] = None
    products: Optional[List[StrictStr]] = None
    name: Optional[StrictStr] = None
    __properties: ClassVar[List[str]] = [
        "valid_until",
        "usage_limit",
        "products",
        "name",
    ]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of CouponUpdate from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # set to None if valid_until (nullable) is None
        # and model_fields_set contains the field
        if self.valid_until is None and "valid_until" in self.model_fields_set:
            _dict["valid_until"] = None

        # set to None if usage_limit (nullable) is None
        # and model_fields_set contains the field
        if self.usage_limit is None and "usage_limit" in self.model_fields_set:
            _dict["usage_limit"] = None

        # set to None if products (nullable) is None
        # and model_fields_set contains the field
        if self.products is None and "products" in self.model_fields_set:
            _dict["products"] = None

        # set to None if name (nullable) is None
        # and model_fields_set contains the field
        if self.name is None and "name" in self.model_fields_set:
            _dict["name"] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of CouponUpdate from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "valid_until": obj.get("valid_until"),
                "usage_limit": obj.get("usage_limit"),
                "products": obj.get("products"),
                "name": obj.get("name"),
            }
        )
        return _obj



from __future__ import annotations

import warnings
from typing import TYPE_CHECKING, Optional

from typing_extensions import deprecated

from crypticorn._internal.warnings import CrypticornDeprecatedSince31
from crypticorn.hive.client import (
    ApiClient,
    Configuration,
    ModelsApi,
    StatusApi,
)
from crypticorn.hive.wrapper import DataApiWrapper  # wraps DataApi

if TYPE_CHECKING:
    from aiohttp import ClientSession


class HiveClient(DataApiWrapper, ModelsApi, StatusApi):
    """
    A client for interacting with the Crypticorn Hive API.
    """

    config_class = Configuration

    def __init__(
        self,
        config: Configuration,
        http_client: Optional[ClientSession] = None,
        is_sync: bool = False,
    ):
        self.config = config
        self.base_client = ApiClient(configuration=self.config)
        if http_client is not None:
            self.base_client.rest_client.pool_manager = http_client
        # Pass sync context to REST client for proper session management
        self.base_client.rest_client.is_sync = is_sync
        super().__init__(self.base_client, is_sync=is_sync)
        # TODO: remove everything below this line in v4
        self._models = ModelsApi(self.base_client, is_sync=is_sync)
        self._data = DataApiWrapper(self.base_client, is_sync=is_sync)
        self._status = StatusApi(self.base_client, is_sync=is_sync)

    @property
    @deprecated(
        "Accessing hive.models is deprecated. Use direct method calls on hive instead (e.g., hive.create_model())"
    )
    def models(self):
        warnings.warn(
            "Accessing hive.models is deprecated. Use direct method calls on hive instead (e.g., hive.create_model())",
            category=CrypticornDeprecatedSince31,
        )
        return self._models

    @property
    @deprecated(
        "Accessing hive.data is deprecated. Use direct method calls on hive instead (e.g., hive.download_data())"
    )
    def data(self):
        warnings.warn(
            "Accessing hive.data is deprecated. Use direct method calls on hive instead (e.g., hive.download_data())",
            category=CrypticornDeprecatedSince31,
        )
        return self._data

    @property
    @deprecated(
        "Accessing hive.status is deprecated. Use direct method calls on hive instead (e.g., hive.ping())"
    )
    def status(self):
        warnings.warn(
            "Accessing hive.status is deprecated. Use direct method calls on hive instead (e.g., hive.ping())",
            category=CrypticornDeprecatedSince31,
        )
        return self._status



from __future__ import annotations

import warnings
from typing import TYPE_CHECKING, Optional

from typing_extensions import deprecated

from crypticorn._internal.warnings import CrypticornDeprecatedSince31
from crypticorn.auth import (
    AdminApi,
    ApiClient,
    AuthApi,
    Configuration,
    UserApi,
    WalletApi,
)

if TYPE_CHECKING:
    from aiohttp import ClientSession


class AuthClient(AdminApi, AuthApi, UserApi, WalletApi):
    """
    A client for interacting with the Crypticorn Auth API.
    """

    config_class = Configuration

    def __init__(
        self,
        config: Configuration,
        http_client: Optional[ClientSession] = None,
        is_sync: bool = False,
    ):
        self.config = config
        self.base_client = ApiClient(configuration=self.config)
        if http_client is not None:
            self.base_client.rest_client.pool_manager = http_client
        # Pass sync context to REST client for proper session management
        self.base_client.rest_client.is_sync = is_sync
        super().__init__(self.base_client, is_sync=is_sync)
        # TODO: remove everything below this line in v4
        self._admin = AdminApi(self.base_client, is_sync=is_sync)
        self._user = UserApi(self.base_client, is_sync=is_sync)
        self._wallet = WalletApi(self.base_client, is_sync=is_sync)
        self._login = AuthApi(self.base_client, is_sync=is_sync)

    @property
    @deprecated(
        "Accessing auth.admin is deprecated. Use direct method calls instead (e.g., auth.get_users())"
    )
    def admin(self):
        warnings.warn(
            "Accessing auth.admin is deprecated. Use direct method calls instead (e.g., auth.get_users())",
            category=CrypticornDeprecatedSince31,
        )
        return self._admin

    @property
    @deprecated(
        "Accessing auth.user is deprecated. Use direct method calls instead (e.g., auth.get_user())"
    )
    def user(self):
        warnings.warn(
            "Accessing auth.user is deprecated. Use direct method calls instead (e.g., auth.get_user())",
            category=CrypticornDeprecatedSince31,
        )
        return self._user

    @property
    @deprecated(
        "Accessing auth.wallet is deprecated. Use direct method calls instead (e.g., auth.add_wallet())"
    )
    def wallet(self):
        warnings.warn(
            "Accessing auth.wallet is deprecated. Use direct method calls instead (e.g., auth.add_wallet())",
            category=CrypticornDeprecatedSince31,
        )
        return self._wallet

    @property
    @deprecated(
        "Accessing auth.login is deprecated. Use direct method calls instead (e.g., auth.authorize_user())"
    )
    def login(self):
        warnings.warn(
            "Accessing auth.login is deprecated. Use direct method calls instead (e.g., auth.authorize_user())",
            category=CrypticornDeprecatedSince31,
        )
        return self._login



# coding: utf-8

"""
Trading API

API for automated trading and exchange interface. This API is used to trade on the exchange and manage bots, API keys, orders, and more.

The version of the OpenAPI document: 2.0.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, StrictStr, field_validator
from typing import Any, ClassVar, Dict, List, Optional
from typing import Set
from typing_extensions import Self


class BotUpdate(BaseModel):
    """
    Trading bot model for API update operations. Fields cannot be unset.
    """  # noqa: E501

    status: Optional[StrictStr] = None
    __properties: ClassVar[List[str]] = ["status"]

    @field_validator("status")
    def status_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in set(["running", "stopping", "stopped", "deleted"]):
            raise ValueError(
                "must be one of enum values ('running', 'stopping', 'stopped', 'deleted')"
            )
        return value

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of BotUpdate from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # set to None if status (nullable) is None
        # and model_fields_set contains the field
        if self.status is None and "status" in self.model_fields_set:
            _dict["status"] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of BotUpdate from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({"status": obj.get("status")})
        return _obj



# coding: utf-8

"""
Payment API

API for accepting payments and storing subscriptions

The version of the OpenAPI document: 1.0.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

from typing import Any, Awaitable, Dict, List, Optional, Tuple, Union

from asgiref.sync import async_to_sync
from pydantic import Field, StrictBool, StrictFloat, StrictInt, StrictStr, validate_call
from typing_extensions import Annotated

from crypticorn.pay.client.api_client import ApiClient, RequestSerialized
from crypticorn.pay.client.api_response import ApiResponse
from crypticorn.pay.client.models.access_threshold import AccessThreshold
from crypticorn.pay.client.models.scopes_info import ScopesInfo
from crypticorn.pay.client.models.user_balance import UserBalance
from crypticorn.pay.client.rest import RESTResponseType


class AccessApi:
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None, is_sync: bool = False) -> None:
        if api_client is None:
            api_client = ApiClient.get_default()
        self.api_client = api_client
        self.is_sync = is_sync

    @validate_call
    def get_balances(
        self,
        user_id: Annotated[
            Optional[StrictStr],
            Field(
                description="Wallet address to check balance for. Overrides the authenticated user if provided and the user is an admin."
            ),
        ] = None,
        force: Annotated[
            Optional[StrictBool],
            Field(description="Force live fetch and update cache."),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> Union[UserBalance, Awaitable[UserBalance]]:
        """Get Balances"""
        if self.is_sync:
            return self._get_balances_sync(
                user_id=user_id,
                force=force,
                _request_timeout=_request_timeout,
                _request_auth=_request_auth,
                _content_type=_content_type,
                _headers=_headers,
                _host_index=_host_index,
            )

        else:
            return self._get_balances_async(
                user_id=user_id,
                force=force,
                _request_timeout=_request_timeout,
                _request_auth=_request_auth,
                _content_type=_content_type,
                _headers=_headers,
                _host_index=_host_index,
            )

    @validate_call
    def get_balances_with_http_info(
        self,
        user_id: Annotated[
            Optional[StrictStr],
            Field(
                description="Wallet address to check balance for. Overrides the authenticated user if provided and the user is an admin."
            ),
        ] = None,
        force: Annotated[
            Optional[StrictBool],
            Field(description="Force live fetch and update cache."),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> Union[ApiResponse[UserBalance], Awaitable[ApiResponse[UserBalance]]]:
        """Get Balances with HTTP info"""
        if self.is_sync:
            return self._get_balances_sync_with_http_info(
                user_id=user_id,
                force=force,
                _request_timeout=_request_timeout,
                _request_auth=_request_auth,
                _content_type=_content_type,
                _headers=_headers,
                _host_index=_host_index,
            )

        else:
            return self._get_balances_async_with_http_info(
                user_id=user_id,
                force=force,
                _request_timeout=_request_timeout,
                _request_auth=_request_auth,
                _content_type=_content_type,
                _headers=_headers,
                _host_index=_host_index,
            )

    @validate_call
    def get_balances_without_preload_content(
        self,
        user_id: Annotated[
            Optional[StrictStr],
            Field(
                description="Wallet address to check balance for. Overrides the authenticated user if provided and the user is an admin."
            ),
        ] = None,
        force: Annotated[
            Optional[StrictBool],
            Field(description="Force live fetch and update cache."),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> Union[RESTResponseType, Awaitable[RESTResponseType]]:
        """Get Balances without preloading content"""
        if self.is_sync:
            return self._get_balances_sync_without_preload_content(
                user_id=user_id,
                force=force,
                _request_timeout=_request_timeout,
                _request_auth=_request_auth,
                _content_type=_content_type,
                _headers=_headers,
                _host_index=_host_index,
            )

        else:
            return self._get_balances_async_without_preload_content(
                user_id=user_id,
                force=force,
                _request_timeout=_request_timeout,
                _request_auth=_request_auth,
                _content_type=_content_type,
                _headers=_headers,
                _host_index=_host_index,
            )

    # Private async implementation methods
    @validate_call
    async def _get_balances_async(
        self,
        user_id: Annotated[
            Optional[StrictStr],
            Field(
                description="Wallet address to check balance for. Overrides the authenticated user if provided and the user is an admin."
            ),
        ] = None,
        force: Annotated[
            Optional[StrictBool],
            Field(description="Force live fetch and update cache."),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> UserBalance:
        """Get Balances

        Get user's AIC balance and staking information for each connected wallet.

        :param user_id: Wallet address to check balance for. Overrides the authenticated user if provided and the user is an admin.
        :type user_id: str
        :param force: Force live fetch and update cache.
        :type force: bool
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._get_balances_serialize(
            user_id=user_id,
            force=force,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "UserBalance",
        }

        response_data = await self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        await response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data

    @validate_call
    async def _get_balances_async_with_http_info(
        self,
        user_id: Annotated[
            Optional[StrictStr],
            Field(
                description="Wallet address to check balance for. Overrides the authenticated user if provided and the user is an admin."
            ),
        ] = None,
        force: Annotated[
            Optional[StrictBool],
            Field(description="Force live fetch and update cache."),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[UserBalance]:
        """Get Balances

        Get user's AIC balance and staking information for each connected wallet.

        :param user_id: Wallet address to check balance for. Overrides the authenticated user if provided and the user is an admin.
        :type user_id: str
        :param force: Force live fetch and update cache.
        :type force: bool
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._get_balances_serialize(
            user_id=user_id,
            force=force,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "UserBalance",
        }

        response_data = await self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        await response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data, response_types_map=_response_types_map
        )

    @validate_call
    async def _get_balances_async_without_preload_content(
        self,
        user_id: Annotated[
            Optional[StrictStr],
            Field(
                description="Wallet address to check balance for. Overrides the authenticated user if provided and the user is an admin."
            ),
        ] = None,
        force: Annotated[
            Optional[StrictBool],
            Field(description="Force live fetch and update cache."),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Get Balances

        Get user's AIC balance and staking information for each connected wallet.

        :param user_id: Wallet address to check balance for. Overrides the authenticated user if provided and the user is an admin.
        :type user_id: str
        :param force: Force live fetch and update cache.
        :type force: bool
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._get_balances_serialize(
            user_id=user_id,
            force=force,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "UserBalance",
        }

        response_data = await self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        return response_data

    # Private sync implementation methods
    @validate_call
    def _get_balances_sync(
        self,
        user_id: Annotated[
            Optional[StrictStr],
            Field(
                description="Wallet address to check balance for. Overrides the authenticated user if provided and the user is an admin."
            ),
        ] = None,
        force: Annotated[
            Optional[StrictBool],
            Field(description="Force live fetch and update cache."),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> UserBalance:
        """Synchronous version of get_balances"""
        return async_to_sync(self._get_balances_async)(
            user_id=user_id,
            force=force,
            _request_timeout=_request_timeout,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

    @validate_call
    def _get_balances_sync_with_http_info(
        self,
        user_id: Annotated[
            Optional[StrictStr],
            Field(
                description="Wallet address to check balance for. Overrides the authenticated user if provided and the user is an admin."
            ),
        ] = None,
        force: Annotated[
            Optional[StrictBool],
            Field(description="Force live fetch and update cache."),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[UserBalance]:
        """Synchronous version of get_balances_with_http_info"""
        return async_to_sync(self._get_balances_async_with_http_info)(
            user_id=user_id,
            force=force,
            _request_timeout=_request_timeout,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

    @validate_call
    def _get_balances_sync_without_preload_content(
        self,
        user_id: Annotated[
            Optional[StrictStr],
            Field(
                description="Wallet address to check balance for. Overrides the authenticated user if provided and the user is an admin."
            ),
        ] = None,
        force: Annotated[
            Optional[StrictBool],
            Field(description="Force live fetch and update cache."),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Synchronous version of get_balances_without_preload_content"""
        return async_to_sync(self._get_balances_async_without_preload_content)(
            user_id=user_id,
            force=force,
            _request_timeout=_request_timeout,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

    def _get_balances_serialize(
        self,
        user_id,
        force,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:
        _host = None

        _collection_formats: Dict[str, str] = {}

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        if user_id is not None:
            _query_params.append(("user_id", user_id))

        if force is not None:
            _query_params.append(("force", force))

        # process the header parameters
        # process the form parameters
        # process the body parameter

        # set the HTTP header `Accept`
        if "Accept" not in _header_params:
            _header_params["Accept"] = self.api_client.select_header_accept(
                ["application/json"]
            )

        # authentication setting
        _auth_settings: List[str] = ["APIKeyHeader", "HTTPBearer"]

        return self.api_client.param_serialize(
            method="GET",
            resource_path="/access/balances",
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth,
        )

    @validate_call
    def get_scopes_info(
        self,
        user_id: Annotated[StrictStr, Field(description="User ID to get scopes for")],
        force: Annotated[
            Optional[StrictBool],
            Field(description="Force live fetch and update cache."),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> Union[ScopesInfo, Awaitable[ScopesInfo]]:
        """Get Accessible Scopes"""
        if self.is_sync:
            return self._get_scopes_info_sync(
                user_id=user_id,
                force=force,
                _request_timeout=_request_timeout,
                _request_auth=_request_auth,
                _content_type=_content_type,
                _headers=_headers,
                _host_index=_host_index,
            )

        else:
            return self._get_scopes_info_async(
                user_id=user_id,
                force=force,
                _request_timeout=_request_timeout,
                _request_auth=_request_auth,
                _content_type=_content_type,
                _headers=_headers,
                _host_index=_host_index,
            )

    @validate_call
    def get_scopes_info_with_http_info(
        self,
        user_id: Annotated[StrictStr, Field(description="User ID to get scopes for")],
        force: Annotated[
            Optional[StrictBool],
            Field(description="Force live fetch and update cache."),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> Union[ApiResponse[ScopesInfo], Awaitable[ApiResponse[ScopesInfo]]]:
        """Get Accessible Scopes with HTTP info"""
        if self.is_sync:
            return self._get_scopes_info_sync_with_http_info(
                user_id=user_id,
                force=force,
                _request_timeout=_request_timeout,
                _request_auth=_request_auth,
                _content_type=_content_type,
                _headers=_headers,
                _host_index=_host_index,
            )

        else:
            return self._get_scopes_info_async_with_http_info(
                user_id=user_id,
                force=force,
                _request_timeout=_request_timeout,
                _request_auth=_request_auth,
                _content_type=_content_type,
                _headers=_headers,
                _host_index=_host_index,
            )

    @validate_call
    def get_scopes_info_without_preload_content(
        self,
        user_id: Annotated[StrictStr, Field(description="User ID to get scopes for")],
        force: Annotated[
            Optional[StrictBool],
            Field(description="Force live fetch and update cache."),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> Union[RESTResponseType, Awaitable[RESTResponseType]]:
        """Get Accessible Scopes without preloading content"""
        if self.is_sync:
            return self._get_scopes_info_sync_without_preload_content(
                user_id=user_id,
                force=force,
                _request_timeout=_request_timeout,
                _request_auth=_request_auth,
                _content_type=_content_type,
                _headers=_headers,
                _host_index=_host_index,
            )

        else:
            return self._get_scopes_info_async_without_preload_content(
                user_id=user_id,
                force=force,
                _request_timeout=_request_timeout,
                _request_auth=_request_auth,
                _content_type=_content_type,
                _headers=_headers,
                _host_index=_host_index,
            )

    # Private async implementation methods
    @validate_call
    async def _get_scopes_info_async(
        self,
        user_id: Annotated[StrictStr, Field(description="User ID to get scopes for")],
        force: Annotated[
            Optional[StrictBool],
            Field(description="Force live fetch and update cache."),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ScopesInfo:
        """Get Accessible Scopes

        Core endpoint: Returns all access scopes a user has (currently or in the last 7 days), and detailed info for each access method. - `scopes`: List of currently active scopes (user has access right now). - `info`: List of all access records (active and recently expired), for allowlist, subscription, and balance-based access.

        :param user_id: User ID to get scopes for (required)
        :type user_id: str
        :param force: Force live fetch and update cache.
        :type force: bool
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._get_scopes_info_serialize(
            user_id=user_id,
            force=force,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "ScopesInfo",
        }

        response_data = await self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        await response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data

    @validate_call
    async def _get_scopes_info_async_with_http_info(
        self,
        user_id: Annotated[StrictStr, Field(description="User ID to get scopes for")],
        force: Annotated[
            Optional[StrictBool],
            Field(description="Force live fetch and update cache."),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[ScopesInfo]:
        """Get Accessible Scopes

        Core endpoint: Returns all access scopes a user has (currently or in the last 7 days), and detailed info for each access method. - `scopes`: List of currently active scopes (user has access right now). - `info`: List of all access records (active and recently expired), for allowlist, subscription, and balance-based access.

        :param user_id: User ID to get scopes for (required)
        :type user_id: str
        :param force: Force live fetch and update cache.
        :type force: bool
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._get_scopes_info_serialize(
            user_id=user_id,
            force=force,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "ScopesInfo",
        }

        response_data = await self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        await response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data, response_types_map=_response_types_map
        )

    @validate_call
    async def _get_scopes_info_async_without_preload_content(
        self,
        user_id: Annotated[StrictStr, Field(description="User ID to get scopes for")],
        force: Annotated[
            Optional[StrictBool],
            Field(description="Force live fetch and update cache."),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Get Accessible Scopes

        Core endpoint: Returns all access scopes a user has (currently or in the last 7 days), and detailed info for each access method. - `scopes`: List of currently active scopes (user has access right now). - `info`: List of all access records (active and recently expired), for allowlist, subscription, and balance-based access.

        :param user_id: User ID to get scopes for (required)
        :type user_id: str
        :param force: Force live fetch and update cache.
        :type force: bool
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._get_scopes_info_serialize(
            user_id=user_id,
            force=force,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "ScopesInfo",
        }

        response_data = await self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        return response_data

    # Private sync implementation methods
    @validate_call
    def _get_scopes_info_sync(
        self,
        user_id: Annotated[StrictStr, Field(description="User ID to get scopes for")],
        force: Annotated[
            Optional[StrictBool],
            Field(description="Force live fetch and update cache."),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ScopesInfo:
        """Synchronous version of get_scopes_info"""
        return async_to_sync(self._get_scopes_info_async)(
            user_id=user_id,
            force=force,
            _request_timeout=_request_timeout,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

    @validate_call
    def _get_scopes_info_sync_with_http_info(
        self,
        user_id: Annotated[StrictStr, Field(description="User ID to get scopes for")],
        force: Annotated[
            Optional[StrictBool],
            Field(description="Force live fetch and update cache."),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[ScopesInfo]:
        """Synchronous version of get_scopes_info_with_http_info"""
        return async_to_sync(self._get_scopes_info_async_with_http_info)(
            user_id=user_id,
            force=force,
            _request_timeout=_request_timeout,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

    @validate_call
    def _get_scopes_info_sync_without_preload_content(
        self,
        user_id: Annotated[StrictStr, Field(description="User ID to get scopes for")],
        force: Annotated[
            Optional[StrictBool],
            Field(description="Force live fetch and update cache."),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Synchronous version of get_scopes_info_without_preload_content"""
        return async_to_sync(self._get_scopes_info_async_without_preload_content)(
            user_id=user_id,
            force=force,
            _request_timeout=_request_timeout,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

    def _get_scopes_info_serialize(
        self,
        user_id,
        force,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:
        _host = None

        _collection_formats: Dict[str, str] = {}

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        if user_id is not None:
            _query_params.append(("user_id", user_id))

        if force is not None:
            _query_params.append(("force", force))

        # process the header parameters
        # process the form parameters
        # process the body parameter

        # set the HTTP header `Accept`
        if "Accept" not in _header_params:
            _header_params["Accept"] = self.api_client.select_header_accept(
                ["application/json"]
            )

        # authentication setting
        _auth_settings: List[str] = []

        return self.api_client.param_serialize(
            method="GET",
            resource_path="/access/scopes/info",
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth,
        )

    @validate_call
    def get_thresholds(
        self,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> Union[List[AccessThreshold], Awaitable[List[AccessThreshold]]]:
        """Get Thresholds"""
        if self.is_sync:
            return self._get_thresholds_sync(
                _request_timeout=_request_timeout,
                _request_auth=_request_auth,
                _content_type=_content_type,
                _headers=_headers,
                _host_index=_host_index,
            )

        else:
            return self._get_thresholds_async(
                _request_timeout=_request_timeout,
                _request_auth=_request_auth,
                _content_type=_content_type,
                _headers=_headers,
                _host_index=_host_index,
            )

    @validate_call
    def get_thresholds_with_http_info(
        self,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> Union[
        ApiResponse[List[AccessThreshold]],
        Awaitable[ApiResponse[List[AccessThreshold]]],
    ]:
        """Get Thresholds with HTTP info"""
        if self.is_sync:
            return self._get_thresholds_sync_with_http_info(
                _request_timeout=_request_timeout,
                _request_auth=_request_auth,
                _content_type=_content_type,
                _headers=_headers,
                _host_index=_host_index,
            )

        else:
            return self._get_thresholds_async_with_http_info(
                _request_timeout=_request_timeout,
                _request_auth=_request_auth,
                _content_type=_content_type,
                _headers=_headers,
                _host_index=_host_index,
            )

    @validate_call
    def get_thresholds_without_preload_content(
        self,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> Union[RESTResponseType, Awaitable[RESTResponseType]]:
        """Get Thresholds without preloading content"""
        if self.is_sync:
            return self._get_thresholds_sync_without_preload_content(
                _request_timeout=_request_timeout,
                _request_auth=_request_auth,
                _content_type=_content_type,
                _headers=_headers,
                _host_index=_host_index,
            )

        else:
            return self._get_thresholds_async_without_preload_content(
                _request_timeout=_request_timeout,
                _request_auth=_request_auth,
                _content_type=_content_type,
                _headers=_headers,
                _host_index=_host_index,
            )

    # Private async implementation methods
    @validate_call
    async def _get_thresholds_async(
        self,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> List[AccessThreshold]:
        """Get Thresholds

        Get the access thresholds for the scopes.

        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._get_thresholds_serialize(
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "List[AccessThreshold]",
        }

        response_data = await self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        await response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data

    @validate_call
    async def _get_thresholds_async_with_http_info(
        self,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[List[AccessThreshold]]:
        """Get Thresholds

        Get the access thresholds for the scopes.

        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._get_thresholds_serialize(
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "List[AccessThreshold]",
        }

        response_data = await self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        await response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data, response_types_map=_response_types_map
        )

    @validate_call
    async def _get_thresholds_async_without_preload_content(
        self,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Get Thresholds

        Get the access thresholds for the scopes.

        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._get_thresholds_serialize(
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "List[AccessThreshold]",
        }

        response_data = await self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        return response_data

    # Private sync implementation methods
    @validate_call
    def _get_thresholds_sync(
        self,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> List[AccessThreshold]:
        """Synchronous version of get_thresholds"""
        return async_to_sync(self._get_thresholds_async)(
            _request_timeout=_request_timeout,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

    @validate_call
    def _get_thresholds_sync_with_http_info(
        self,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[List[AccessThreshold]]:
        """Synchronous version of get_thresholds_with_http_info"""
        return async_to_sync(self._get_thresholds_async_with_http_info)(
            _request_timeout=_request_timeout,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

    @validate_call
    def _get_thresholds_sync_without_preload_content(
        self,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Synchronous version of get_thresholds_without_preload_content"""
        return async_to_sync(self._get_thresholds_async_without_preload_content)(
            _request_timeout=_request_timeout,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

    def _get_thresholds_serialize(
        self,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:
        _host = None

        _collection_formats: Dict[str, str] = {}

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter

        # set the HTTP header `Accept`
        if "Accept" not in _header_params:
            _header_params["Accept"] = self.api_client.select_header_accept(
                ["application/json"]
            )

        # authentication setting
        _auth_settings: List[str] = []

        return self.api_client.param_serialize(
            method="GET",
            resource_path="/access/thresholds",
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth,
        )



from __future__ import annotations

from typing import TYPE_CHECKING, Awaitable, Optional, Union

from pydantic import StrictInt, StrictStr

from crypticorn._internal.utils import optional_import
from crypticorn.metrics import (
    ExchangesApi,
    MarketcapApi,
    MarketType,
    TimeInterval,
    TokensApi,
    TradingStatus,
)

try:
    import pandas as pd
except ImportError:
    pd = None

if TYPE_CHECKING:
    pass


class MarketcapApiWrapper(MarketcapApi):
    """
    A wrapper for the MarketcapApi class.
    """

    def get_marketcap_symbols_fmt(
        self,
        start_timestamp: Optional[StrictInt] = None,
        end_timestamp: Optional[StrictInt] = None,
        interval: Optional[StrictStr] = None,
        market: Optional[MarketType] = None,
        exchange: Optional[StrictStr] = None,
        **kwargs,
    ) -> Union["pd.DataFrame", Awaitable["pd.DataFrame"]]:
        """
        Get the marketcap symbols in a pandas dataframe
        Works in both sync and async contexts.
        """
        if self.is_sync:
            return self._get_marketcap_symbols_fmt_sync(
                start_timestamp=start_timestamp,
                end_timestamp=end_timestamp,
                interval=interval,
                market=market,
                exchange=exchange,
                **kwargs,
            )
        else:
            return self._get_marketcap_symbols_fmt_async(
                start_timestamp=start_timestamp,
                end_timestamp=end_timestamp,
                interval=interval,
                market=market,
                exchange=exchange,
                **kwargs,
            )

    def _get_marketcap_symbols_fmt_sync(
        self,
        start_timestamp: Optional[StrictInt] = None,
        end_timestamp: Optional[StrictInt] = None,
        interval: Optional[StrictStr] = None,
        market: Optional[MarketType] = None,
        exchange: Optional[StrictStr] = None,
        **kwargs,
    ) -> "pd.DataFrame":
        """
        Get the marketcap symbols in a pandas dataframe (sync version)
        """
        pd = optional_import("pandas", "extra")
        response = self._get_marketcap_symbols_sync(
            start_timestamp=start_timestamp,
            end_timestamp=end_timestamp,
            interval=interval,
            market=market,
            exchange=exchange,
            **kwargs,
        )
        rows = []
        for item in response:
            row: dict[Union[int, str], Union[int, str, None]] = {
                "timestamp": item.timestamp
            }
            row.update({i + 1: sym for i, sym in enumerate(item.symbols)})
            rows.append(row)
        df = pd.DataFrame(rows)
        return df

    async def _get_marketcap_symbols_fmt_async(
        self,
        start_timestamp: Optional[StrictInt] = None,
        end_timestamp: Optional[StrictInt] = None,
        interval: Optional[StrictStr] = None,
        market: Optional[MarketType] = None,
        exchange: Optional[StrictStr] = None,
        **kwargs,
    ) -> "pd.DataFrame":
        """
        Get the marketcap symbols in a pandas dataframe (async version)
        """
        pd = optional_import("pandas", "extra")
        response = await self._get_marketcap_symbols_async(
            start_timestamp=start_timestamp,
            end_timestamp=end_timestamp,
            interval=interval,
            market=market,
            exchange=exchange,
            **kwargs,
        )
        rows = []
        for item in response:
            row: dict[Union[int, str], Union[int, str, None]] = {
                "timestamp": item.timestamp
            }
            row.update({i + 1: sym for i, sym in enumerate(item.symbols)})
            rows.append(row)
        df = pd.DataFrame(rows)
        return df


class TokensApiWrapper(TokensApi):
    """
    A wrapper for the TokensApi class.
    """

    def get_stable_tokens_fmt(
        self, **kwargs
    ) -> Union["pd.DataFrame", Awaitable["pd.DataFrame"]]:
        """
        Get the tokens in a pandas dataframe
        Works in both sync and async contexts.
        """
        if self.is_sync:
            return self._get_stable_tokens_fmt_sync(**kwargs)
        else:
            return self._get_stable_tokens_fmt_async(**kwargs)

    def _get_stable_tokens_fmt_sync(self, **kwargs) -> "pd.DataFrame":
        """
        Get the tokens in a pandas dataframe (sync version)
        """
        pd = optional_import("pandas", "extra")
        response = self._get_stable_tokens_sync(**kwargs)
        return pd.DataFrame(response)

    async def _get_stable_tokens_fmt_async(self, **kwargs) -> "pd.DataFrame":
        """
        Get the tokens in a pandas dataframe (async version)
        """
        pd = optional_import("pandas", "extra")
        response = await self._get_stable_tokens_async(**kwargs)
        return pd.DataFrame(response)

    def get_wrapped_tokens_fmt(
        self, **kwargs
    ) -> Union["pd.DataFrame", Awaitable["pd.DataFrame"]]:
        """
        Get the wrapped tokens in a pandas dataframe
        Works in both sync and async contexts.
        """
        if self.is_sync:
            return self._get_wrapped_tokens_fmt_sync(**kwargs)
        else:
            return self._get_wrapped_tokens_fmt_async(**kwargs)

    def _get_wrapped_tokens_fmt_sync(self, **kwargs) -> "pd.DataFrame":
        """
        Get the wrapped tokens in a pandas dataframe (sync version)
        """
        pd = optional_import("pandas", "extra")
        response = self._get_wrapped_tokens_sync(**kwargs)
        return pd.DataFrame(response)

    async def _get_wrapped_tokens_fmt_async(self, **kwargs) -> "pd.DataFrame":
        """
        Get the wrapped tokens in a pandas dataframe (async version)
        """
        pd = optional_import("pandas", "extra")
        response = await self._get_wrapped_tokens_async(**kwargs)
        return pd.DataFrame(response)


class ExchangesApiWrapper(ExchangesApi):
    """
    A wrapper for the ExchangesApi class.
    """

    def get_available_exchanges_fmt(
        self,
        market: MarketType,
        symbol: StrictStr,
        interval: Optional[TimeInterval] = None,
        start_timestamp: Optional[StrictInt] = None,
        end_timestamp: Optional[StrictInt] = None,
        quote_currency: Optional[StrictStr] = None,
        status: Optional[TradingStatus] = None,
        **kwargs,
    ) -> Union["pd.DataFrame", Awaitable["pd.DataFrame"]]:
        """
        Get the exchanges in a pandas dataframe
        Works in both sync and async contexts.
        """
        if self.is_sync:
            return self._get_available_exchanges_fmt_sync(
                market=market,
                symbol=symbol,
                interval=interval,
                start_timestamp=start_timestamp,
                end_timestamp=end_timestamp,
                quote_currency=quote_currency,
                status=status,
                **kwargs,
            )
        else:
            return self._get_available_exchanges_fmt_async(
                market=market,
                symbol=symbol,
                interval=interval,
                start_timestamp=start_timestamp,
                end_timestamp=end_timestamp,
                quote_currency=quote_currency,
                status=status,
                **kwargs,
            )

    def _get_available_exchanges_fmt_sync(
        self,
        market: MarketType,
        symbol: StrictStr,
        interval: Optional[TimeInterval] = None,
        start_timestamp: Optional[StrictInt] = None,
        end_timestamp: Optional[StrictInt] = None,
        quote_currency: Optional[StrictStr] = None,
        status: Optional[TradingStatus] = None,
        **kwargs,
    ) -> "pd.DataFrame":  # noqa: F821
        """
        Get the exchanges in a pandas dataframe (sync version)
        """
        pd = optional_import("pandas", "extra")
        response = self._get_available_exchanges_sync(
            market=market,
            symbol=symbol,
            interval=interval,
            start_timestamp=start_timestamp,
            end_timestamp=end_timestamp,
            quote_currency=quote_currency,
            status=status,
            **kwargs,
        )

        # Create list of dictionaries with timestamp and flattened exchange data
        rows = []
        for item in response:
            row: dict[str, Union[int, bool]] = {"timestamp": item.timestamp}
            row.update(
                item.exchanges
            )  # This spreads the exchanges dict into individual columns
            rows.append(row)

        df = pd.DataFrame(rows)
        return df

    async def _get_available_exchanges_fmt_async(
        self,
        market: MarketType,
        symbol: StrictStr,
        interval: Optional[TimeInterval] = None,
        start_timestamp: Optional[StrictInt] = None,
        end_timestamp: Optional[StrictInt] = None,
        quote_currency: Optional[StrictStr] = None,
        status: Optional[TradingStatus] = None,
        **kwargs,
    ) -> "pd.DataFrame":  # noqa: F821
        """
        Get the exchanges in a pandas dataframe (async version)
        """
        pd = optional_import("pandas", "extra")
        response = await self._get_available_exchanges_async(
            market=market,
            symbol=symbol,
            interval=interval,
            start_timestamp=start_timestamp,
            end_timestamp=end_timestamp,
            quote_currency=quote_currency,
            status=status,
            **kwargs,
        )

        # Create list of dictionaries with timestamp and flattened exchange data
        rows = []
        for item in response:
            row: dict[str, Union[int, bool]] = {"timestamp": item.timestamp}
            row.update(
                item.exchanges
            )  # This spreads the exchanges dict into individual columns
            rows.append(row)

        df = pd.DataFrame(rows)
        return df



from __future__ import annotations

import warnings
from typing import TYPE_CHECKING, Optional

from typing_extensions import deprecated

from crypticorn._internal.warnings import CrypticornDeprecatedSince31
from crypticorn.notification import (
    ApiClient,
    Configuration,
    NotificationsApi,
    SettingsApi,
    StatusApi,
    TemplatesApi,
)

if TYPE_CHECKING:
    from aiohttp import ClientSession


class NotificationClient(NotificationsApi, SettingsApi, StatusApi, TemplatesApi):
    """
    A client for interacting with the Crypticorn Notification API.
    """

    config_class = Configuration

    def __init__(
        self,
        config: Configuration,
        http_client: Optional[ClientSession] = None,
        is_sync: bool = False,
    ):
        self.config = config
        self.base_client = ApiClient(configuration=self.config)
        if http_client is not None:
            self.base_client.rest_client.pool_manager = http_client
        # Pass sync context to REST client for proper session management
        self.base_client.rest_client.is_sync = is_sync
        super().__init__(self.base_client, is_sync=is_sync)
        # TODO: remove everything below this line in v4
        self._notifications = NotificationsApi(self.base_client, is_sync=is_sync)
        self._templates = TemplatesApi(self.base_client, is_sync=is_sync)
        self._settings = SettingsApi(self.base_client, is_sync=is_sync)
        self._status = StatusApi(self.base_client, is_sync=is_sync)

    @property
    @deprecated(
        "Accessing notification.notifications is deprecated. Use direct method calls instead (e.g., notification.create_notification())"
    )
    def notifications(self):
        warnings.warn(
            "Accessing notification.notifications is deprecated. Use direct method calls instead (e.g., notification.create_notification())",
            category=CrypticornDeprecatedSince31,
        )
        return self._notifications

    @property
    @deprecated(
        "Accessing notification.templates is deprecated. Use direct method calls instead (e.g., notification.get_templates())"
    )
    def templates(self):
        warnings.warn(
            "Accessing notification.templates is deprecated. Use direct method calls instead (e.g., notification.get_templates())",
            category=CrypticornDeprecatedSince31,
        )
        return self._templates

    @property
    @deprecated(
        "Accessing notification.settings is deprecated. Use direct method calls instead (e.g., notification.get_settings())"
    )
    def settings(self):
        warnings.warn(
            "Accessing notification.settings is deprecated. Use direct method calls instead (e.g., notification.get_settings())",
            category=CrypticornDeprecatedSince31,
        )
        return self._settings

    @property
    @deprecated(
        "Accessing notification.status is deprecated. Use direct method calls instead (e.g., notification.ping())"
    )
    def status(self):
        warnings.warn(
            "Accessing notification.status is deprecated. Use direct method calls instead (e.g., notification.ping())",
            category=CrypticornDeprecatedSince31,
        )
        return self._status



# coding: utf-8

"""
Hive AI API

API for Hive AI model training and evaluation

The version of the OpenAPI document: 1.0.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

from __future__ import annotations

import json
import pprint
import re  # noqa: F401
from typing import Any, ClassVar, Dict, List, Optional, Set

from pydantic import BaseModel, ConfigDict, Field, StrictStr, field_validator
from typing_extensions import Self

from crypticorn.hive.client.models.target import Target


class TargetInfo(BaseModel):
    """
    Metadata about an AI prediction target including type and version availability.
    """  # noqa: E501

    name: Target = Field(description="The name of the target.")
    type: StrictStr = Field(description="The type of the target.")
    version_added: StrictStr = Field(
        description="The data version the target got introduced in."
    )
    version_removed: Optional[StrictStr] = None
    __properties: ClassVar[List[str]] = [
        "name",
        "type",
        "version_added",
        "version_removed",
    ]

    @field_validator("type")
    def type_validate_enum(cls, value):
        """Validates the enum"""
        if value not in set(["continuous", "binary"]):
            raise ValueError("must be one of enum values ('continuous', 'binary')")
        return value

    @field_validator("version_added")
    def version_added_validate_enum(cls, value):
        """Validates the enum"""
        if value not in set(["1.0"]):
            raise ValueError("must be one of enum values ('1.0')")
        return value

    @field_validator("version_removed")
    def version_removed_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in set(["1.0"]):
            raise ValueError("must be one of enum values ('1.0')")
        return value

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of TargetInfo from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # set to None if version_removed (nullable) is None
        # and model_fields_set contains the field
        if self.version_removed is None and "version_removed" in self.model_fields_set:
            _dict["version_removed"] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of TargetInfo from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "name": obj.get("name"),
                "type": obj.get("type"),
                "version_added": obj.get("version_added"),
                "version_removed": obj.get("version_removed"),
            }
        )
        return _obj



from __future__ import annotations

import warnings
from typing import TYPE_CHECKING, Optional

from typing_extensions import deprecated

from crypticorn._internal.warnings import CrypticornDeprecatedSince31
from crypticorn.dex import (
    ApiClient,
    Configuration,
    SignalsApi,
    StatusApi,
)

if TYPE_CHECKING:
    from aiohttp import ClientSession


class DexClient(SignalsApi, StatusApi):
    """
    A client for interacting with the Crypticorn DEX API.
    """

    config_class = Configuration

    def __init__(
        self,
        config: Configuration,
        http_client: Optional[ClientSession] = None,
        is_sync: bool = False,
    ):
        self.config = config
        self.base_client = ApiClient(configuration=self.config)
        if http_client is not None:
            self.base_client.rest_client.pool_manager = http_client
        # Pass sync context to REST client for proper session management
        self.base_client.rest_client.is_sync = is_sync
        super().__init__(self.base_client, is_sync=is_sync)
        # TODO: remove everything below this line in v4
        self._signals = SignalsApi(self.base_client, is_sync=is_sync)
        self._status = StatusApi(self.base_client, is_sync=is_sync)

    @property
    @deprecated(
        "Accessing dex.signals is deprecated. Use direct method calls instead (e.g., dex.get_signals())"
    )
    def signals(self):
        warnings.warn(
            "Accessing dex.signals is deprecated. Use direct method calls instead (e.g., dex.get_signals())",
            category=CrypticornDeprecatedSince31,
        )
        return self._signals

    @property
    @deprecated(
        "Accessing dex.status is deprecated. Use direct method calls instead (e.g., dex.ping())"
    )
    def status(self):
        warnings.warn(
            "Accessing dex.status is deprecated. Use direct method calls instead (e.g., dex.ping())",
            category=CrypticornDeprecatedSince31,
        )
        return self._status



# coding: utf-8

"""
Trading API

API for automated trading and exchange interface. This API is used to trade on the exchange and manage bots, API keys, orders, and more.

The version of the OpenAPI document: 2.0.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictInt, StrictStr, field_validator
from typing import Any, ClassVar, Dict, List, Optional
from typing_extensions import Annotated
from crypticorn.trade.client.models.tpsl_create import TPSLCreate
from crypticorn.trade.client.models.trading_action_type import TradingActionType
from typing import Set
from typing_extensions import Self


class FuturesTradingActionCreate(BaseModel):
    """
    Model for sending futures trading actions to the API
    """  # noqa: E501

    leverage: Optional[Annotated[int, Field(strict=True, ge=1)]] = Field(
        default=1, description="Leverage to use for futures trades. Default is 1."
    )
    margin_mode: Optional[StrictStr] = Field(
        default="isolated",
        description="Margin mode for futures trades. Default is isolated.",
    )
    execution_id: Optional[StrictStr] = None
    open_order_execution_id: Optional[StrictStr] = None
    action_type: TradingActionType = Field(description="The type of action.")
    market_type: Optional[StrictStr] = Field(
        default="futures",
        description="The type of market the action is for. Must be set to futures.",
    )
    strategy_id: StrictStr = Field(description="UID for the strategy.")
    symbol: StrictStr = Field(
        description="Trading symbol or asset pair in format: 'symbol/quote_currency' (see market service for valid symbols)"
    )
    limit_price: Optional[StrictStr] = None
    allocation: StrictStr = Field(
        description="How much of bot's balance to use for the order (for open actions). How much of the reference open order (open_order_execution_id) to close (for close actions). 0=0%, 1=100%."
    )
    take_profit: Optional[List[TPSLCreate]] = None
    stop_loss: Optional[List[TPSLCreate]] = None
    expiry_timestamp: Optional[StrictInt] = None
    __properties: ClassVar[List[str]] = [
        "leverage",
        "margin_mode",
        "execution_id",
        "open_order_execution_id",
        "action_type",
        "market_type",
        "strategy_id",
        "symbol",
        "limit_price",
        "allocation",
        "take_profit",
        "stop_loss",
        "expiry_timestamp",
    ]

    @field_validator("margin_mode")
    def margin_mode_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in set(["isolated", "cross"]):
            raise ValueError("must be one of enum values ('isolated', 'cross')")
        return value

    @field_validator("market_type")
    def market_type_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in set(["spot", "futures"]):
            raise ValueError("must be one of enum values ('spot', 'futures')")
        return value

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of FuturesTradingActionCreate from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in take_profit (list)
        _items = []
        if self.take_profit:
            for _item_take_profit in self.take_profit:
                if _item_take_profit:
                    _items.append(_item_take_profit.to_dict())
            _dict["take_profit"] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in stop_loss (list)
        _items = []
        if self.stop_loss:
            for _item_stop_loss in self.stop_loss:
                if _item_stop_loss:
                    _items.append(_item_stop_loss.to_dict())
            _dict["stop_loss"] = _items
        # set to None if execution_id (nullable) is None
        # and model_fields_set contains the field
        if self.execution_id is None and "execution_id" in self.model_fields_set:
            _dict["execution_id"] = None

        # set to None if open_order_execution_id (nullable) is None
        # and model_fields_set contains the field
        if (
            self.open_order_execution_id is None
            and "open_order_execution_id" in self.model_fields_set
        ):
            _dict["open_order_execution_id"] = None

        # set to None if limit_price (nullable) is None
        # and model_fields_set contains the field
        if self.limit_price is None and "limit_price" in self.model_fields_set:
            _dict["limit_price"] = None

        # set to None if take_profit (nullable) is None
        # and model_fields_set contains the field
        if self.take_profit is None and "take_profit" in self.model_fields_set:
            _dict["take_profit"] = None

        # set to None if stop_loss (nullable) is None
        # and model_fields_set contains the field
        if self.stop_loss is None and "stop_loss" in self.model_fields_set:
            _dict["stop_loss"] = None

        # set to None if expiry_timestamp (nullable) is None
        # and model_fields_set contains the field
        if (
            self.expiry_timestamp is None
            and "expiry_timestamp" in self.model_fields_set
        ):
            _dict["expiry_timestamp"] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of FuturesTradingActionCreate from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "leverage": obj.get("leverage")
                if obj.get("leverage") is not None
                else 1,
                "margin_mode": obj.get("margin_mode")
                if obj.get("margin_mode") is not None
                else "isolated",
                "execution_id": obj.get("execution_id"),
                "open_order_execution_id": obj.get("open_order_execution_id"),
                "action_type": obj.get("action_type"),
                "market_type": obj.get("market_type")
                if obj.get("market_type") is not None
                else "futures",
                "strategy_id": obj.get("strategy_id"),
                "symbol": obj.get("symbol"),
                "limit_price": obj.get("limit_price"),
                "allocation": obj.get("allocation"),
                "take_profit": [
                    TPSLCreate.from_dict(_item) for _item in obj["take_profit"]
                ]
                if obj.get("take_profit") is not None
                else None,
                "stop_loss": [TPSLCreate.from_dict(_item) for _item in obj["stop_loss"]]
                if obj.get("stop_loss") is not None
                else None,
                "expiry_timestamp": obj.get("expiry_timestamp"),
            }
        )
        return _obj
